package org.minecms.dyneco;

import java.sql.*;
import java.text.DecimalFormat;

import org.bukkit.ChatColor;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;

public class EconomyHandler {
	DynEco dyneco;
	private YamlConfiguration itemValues;

	public EconomyHandler(DynEco Dyneco) {
		dyneco = Dyneco;
		itemValues = dyneco.getItemValues();

	}

	/**
	 * 
	 * @param ItemStack
	 *            item A item stack to find the sell price of
	 * @return double The total sell price for the stack
	 * 
	 *         Calculates the sell price of an item stack.
	 */

	public double getCurrentSellValue(ItemStack stack) {
		int d = 1;
		try {
			d = getBoughtTally(stack.getType().name());
		} catch (SQLException e) {
			e.printStackTrace();
		}
		int s = 1;
		try {
			s = getSoldTally(stack.getType().name());
		} catch (SQLException e) {
			e.printStackTrace();
		}
		double multiplier = getMultiplierValue();
		double k = getItemConstant(stack);
		double sell = multiplier / 100 * Math.log10((double) d / s) * k + k;
		int amount = stack.getAmount();
		double total = sell * amount;
		return roundTwoDP(total);
	}

	/**
	 * 
	 * @param ItemStack
	 *            item An item stack to find the buy price of
	 * @return double The total buy price for the stack
	 * 
	 *         Calculates the buy price of an item stack.
	 */
	public double getCurrentBuyValue(ItemStack item) {
		int d = 1;
		try {
			d = getBoughtTally(item.getType().name());
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		int s = 1;
		try {
			s = getSoldTally(item.getType().name());
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		double k = getItemConstant(item);
		double multiplier = getMultiplierValue();
		double tax = getCurrentTax(item.getType().name());
		double total = multiplier / 100 * Math.log10((double) d / s) * k + k;
		int amount = item.getAmount();
		double buy = (double) (total * amount + tax * total * amount);

		return roundTwoDP(buy);
	}

	double roundTwoDP(double d) {
		DecimalFormat twoDForm = new DecimalFormat("#.##");
		return Double.valueOf(twoDForm.format(d));
	}

	public int getSoldTally(String item) throws SQLException {
		ResultSet result = dyneco.mysqlStmt
				.executeQuery("SELECT * FROM `transactions` WHERE `item` = '"
						+ item
						+ "' AND `timestamp` >= DATE_SUB(NOW(), INTERVAL "
						+ dyneco.getConfig().getInt("economy.span")
						+ " DAY) AND `type` = '0'");
		int tally = 0;
		while (result.next()) {
			tally = tally + result.getInt("amount");
		}
		if (tally == 0)
			tally = 1;
		return tally;
	}

	public int getBoughtTally(String item) throws SQLException {
		ResultSet result = dyneco.mysqlStmt
				.executeQuery("SELECT * FROM `transactions` WHERE `item` = '"
						+ item
						+ "' AND `timestamp` >= DATE_SUB(NOW(), INTERVAL "
						+ dyneco.getConfig().getInt("economy.span")
						+ " DAY) AND `type` = '1'");
		int tally = 0;
		while (result.next()) {
			tally = tally + result.getInt("amount");
		}
		if (tally == 0)
			tally = 1;
		return tally;
	}

	public double getItemConstant(ItemStack item) {
		String data = String.valueOf(item.getData().getData());
		System.out.print(data);
		if (data == null) {
			data = "0";
		}
		String path = "worth."
				+ item.getType().name().toLowerCase().replace("_", "") + "."
				+ data;
		if (itemValues.isSet(path))
			return itemValues.getInt(path);
		else if (itemValues.isSet("worth."
				+ item.getType().name().toLowerCase().replace("_", "")))
			return itemValues.getInt("worth."
					+ item.getType().name().toLowerCase().replace("_", ""));
		else
			return 0;
	}

	public double getMultiplierValue() {
		return dyneco.getConfig().getDouble("economy.multiplier");

	}

	private double getCurrentTax(String item) {
		return dyneco.getConfig().getDouble("economy.tax");
	}

	public void databaseGarbageCollection() throws SQLException {
		if (dyneco.getConfig().getBoolean("mysql.purge-enabled")) {
			int result = dyneco.mysqlStmt
					.executeUpdate("DELETE FROM `transactions` WHERE `timestamp` <= DATE_SUB(NOW(), INTERVAL "
							+ dyneco.getConfig().getInt("mysql.purge")
							+ " DAY)");

			System.out.print("[DynEco] Removed " + result
					+ " rows from database");
		}
	}

	public void buyStack(ItemStack stack, Player player) {
		String human_name = stack.getType().name().toLowerCase()
				.replace("_", " ");
		int amount = stack.getAmount();
		if (stack.getAmount() <= 64) {
			double total_cost = getCurrentBuyValue(stack);
			if (dyneco.economy.getBalance(player.getName()) > total_cost) {
				dyneco.LOGGER.info("[DynEco] " + player.getName() + " bought "
						+ stack.getAmount() + " " + stack.getType().name()
						+ " for " + total_cost);
				player.sendMessage(ChatColor.GREEN + "You bought " + amount
						+ " " + ChatColor.AQUA + human_name + ChatColor.GREEN
						+ " for " + ChatColor.RED + "$" + total_cost);
				dyneco.economy.withdrawPlayer(player.getName(), total_cost);
				player.sendMessage(ChatColor.RED
						+ "$"
						+ total_cost
						+ ChatColor.GREEN
						+ " was deducted from your account. Your new balance is "
						+ ChatColor.RED + "$"
						+ dyneco.economy.getBalance(player.getName()));
				player.getInventory().addItem(stack);
				try {
					int result = 0;
					String query = "INSERT INTO `transactions` (`item`,`user`,`amount`,`data`,`type`) VALUES ('"
							+ stack.getType().name()
							+ "','"
							+ player.getName()
							+ "','"
							+ amount
							+ "','"
							+ stack.getData().hashCode() + "','" + 1 + "')";

					result = dyneco.mysqlStmt.executeUpdate(query);
					System.out.print("[DynEco] " + result
							+ " rows were inserted into database.");
				} catch (SQLException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			} else {
				player.sendMessage(ChatColor.RED + "You can not afford to buy "
						+ amount + " " + human_name + "!");
				System.out.print("[DynEco] " + player.getName()
						+ " tried to buy more (" + amount + ") " + human_name
						+ " than they could afford!");
			}

		} else {
			player.sendMessage(ChatColor.RED
					+ "Sorry. You can only buy 64 items at a time.");
			System.out.print("[DynEco] " + player.getName()
					+ " tried to buy more than 64 " + stack.getType().name());
		}
	}

	void sellStack(ItemStack stack, Player player) {
		int amount = stack.getAmount();
		String human_name = stack.getType().name().toLowerCase()
				.replace("_", " ");
		double sell_total = getCurrentSellValue(stack);
		if (player.getInventory().contains(stack)) {
			player.getInventory().removeItem(stack);
			dyneco.LOGGER.info("[DynEco] " + player.getName() + " sold "
					+ amount + " " + human_name + " for " + sell_total);
			dyneco.economy.depositPlayer(player.getName(), sell_total);
			player.sendMessage(ChatColor.GREEN + "You sold " + amount + " "
					+ ChatColor.AQUA + human_name + " for " + ChatColor.RED
					+ "$" + sell_total);
			try {
				int result = 0;
				String query = "INSERT INTO `transactions` (`item`,`user`,`amount`,`data`,`type`) VALUES ('"
						+ stack.getType().name()
						+ "','"
						+ player.getName()
						+ "','"
						+ amount
						+ "','"
						+ stack.getData().hashCode()
						+ "','" + 0 + "')";

				result = dyneco.mysqlStmt.executeUpdate(query);
				System.out.print("[DynEco] " + result
						+ " rows were inserted into database.");
			} catch (SQLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		} else
			player.sendMessage(ChatColor.RED + "You do not have " + amount
					+ " " + human_name + "!!");
	}
}
