package org.minecms.dyneco;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.logging.Level;

import org.bukkit.Material;
import org.bukkit.inventory.ItemStack;

public class ItemDb {
	DynEco dyneco;

	public ItemDb(DynEco dynEco) {
		dyneco = dynEco;
	}

	public ItemStack get(final String id) {
		Map<String, Integer> items = new HashMap<String, Integer>();
		Map<String, Short> durabilities = new HashMap<String, Short>();
		File file = new File(dyneco.getDataFolder(), "items.csv");
		if (!file.canRead()) {
			dyneco.LOGGER.severe("[DynEco] Can not read items.csv file!");
		}
		List<String> lines = new ArrayList<String>();
		try {
			final BufferedReader reader = new BufferedReader(new FileReader(
					file));
			try {
				do {
					final String line = reader.readLine();
					if (line == null) {
						break;
					} else {
						lines.add(line);
					}
				} while (true);
			} finally {
				reader.close();
			}
		} catch (IOException ex) {
			dyneco.LOGGER.log(Level.SEVERE, ex.getMessage(), ex);
			lines = Collections.emptyList();
		}

		durabilities.clear();
		items.clear();

		for (String line : lines) {
			line = line.trim().toLowerCase();
			if (line.length() > 0 && line.charAt(0) == '#') {
				continue;
			}

			final String[] parts = line.split("[^a-z0-9]");
			if (parts.length < 2) {
				continue;
			}

			final int numeric = Integer.parseInt(parts[1]);

			durabilities.put(
					parts[0].toLowerCase(Locale.ENGLISH),
					parts.length > 2 && !parts[2].equals("0") ? Short
							.parseShort(parts[2]) : 0);
			items.put(parts[0].toLowerCase(Locale.ENGLISH), numeric);
		}
		int itemid = 0;
		String itemname = null;
		short metaData = 0;
		if (id.matches("^\\d+[:+',;.]\\d+$")) {
			itemid = Integer.parseInt(id.split("[:+',;.]")[0]);
			metaData = Short.parseShort(id.split("[:+',;.]")[1]);
		} else if (id.matches("^\\d+$")) {
			itemid = Integer.parseInt(id);
		} else if (id.matches("^[^:+',;.]+[:+',;.]\\d+$")) {
			itemname = id.split("[:+',;.]")[0].toLowerCase(Locale.ENGLISH);
			metaData = Short.parseShort(id.split("[:+',;.]")[1]);
		} else {
			itemname = id.toLowerCase(Locale.ENGLISH);
		}

		if (itemname != null) {
			if (items.containsKey(itemname)) {
				itemid = items.get(itemname);
				if (durabilities.containsKey(itemname) && metaData == 0) {
					metaData = durabilities.get(itemname);
				}
			} else if (Material.getMaterial(itemname) != null) {
				itemid = Material.getMaterial(itemname).getId();
				metaData = 0;
			} else {
				// Unknown item name
			}
		}

		final Material mat = Material.getMaterial(itemid);
		if (mat == null) {
			// Unknown item ID
		}
		final ItemStack retval = new ItemStack(mat);
		retval.setAmount(mat.getMaxStackSize());
		retval.setDurability(metaData);
		return retval;
	}
}
